function demoPerfectSim(n)
% demonstrations of some possibilities of the package Context
%
% author: Aurlien Garivier
% reference: Perfect Simulation Scheme For Processes With Long Memory :
%    A ``Coupling Into And From The Past'' Algorithm, by A. Garivier

    if nargin<1, n=1; end
    clf
    if n==1
        demoVLMC
    elseif n==2
        demoExample1
    else
        demoSparseVLMC
    end
end

function demoVLMC
    % build a VLMC on G={1,2} with given contexts and conditional distributions
    P1 = Context.VLMC({[1 1 1]  ,  [2 1 1 ]  ,  [2 1]      ,  [1 2]  , [2 2]     },...
                      {[0.2 0.8],  [0 1]     ,  [0.7 0.3]  ,  [1 0]  , [0.3 0.7] });

    clf;
    subplot(2,1,1); 
    P1.plot();
    title('Representation of the VLMC')

    subplot(2,2,3); 
    P1.plot(true);
    title('Coupling function of the VLMC')

    subplot(2,2,4); 
    u = 0.75;
    P1.transition2(u).plot();
    title(['Transition function for u=', num2str(u)])


    disp('showing before running CIAFTP for X(-4:-1)')
    pause

    [X, t] = P1.CIAFTP(-4, true)

end

function demoExample1
    P2 = Context.Example1();
    u = 0.6; 
    P2.transition(u).plot();
    title(['Transition function for u=', num2str(u)])

    disp('showing before running CIAFTP for X_(-3:-1)')
    pause

    set(0,'RecursionLimit',1e4) % to (partially) avoid problem with large U_k 
    [X, t] = P2.CIAFTP(-2, true)
end

function demoSparseVLMC
    P1 = Context.VLMC({[1 1 1]  ,  [1 2 1 1 ] ,  [1 1 1 2 2 1 1 ] ,  [1 1 2 1 1 2 2 1 1 ] ,  [2 1 2 1 1 2 2 1 1 ] ,  [2 2 1 1 2 2 1 1 ] ,  [2 1 2 2 1 1 ]  ,  [2 2 2 1 1]  , [2 1]     , [1 2]    , [1 2 2]  , [2 2 2] },...
                      {[0.2 0.8],  [0.1 0.9]  ,  [0.2 0.8]        ,  [0.2 0.8]          ,  [0.7 0.3]          ,  [0.1 0.9]          ,  [0.8 0.2]       ,  [0.3 0.7]    , [0.8  0.2], [0.5 0.5], [0.7 0.3], [0.4 0.6]});
    clf;
    subplot(2,1,1); 
    P1.plot();
    title('Representation of the VLMC')

    subplot(2,2,3); 
    P1.plot(true);
    title('Coupling function of the VLMC')

    subplot(2,2,4); 
    u = 0.75;
    P1.transition2(u).plot();
    title(['Transition function for u=', num2str(u)])


    disp('showing before running CIAFTP for X(-4:-1)')
    pause

    [X, t] = P1.CIAFTP(-4, true)

end